<?php
/**
 * Salsa Supporter API Helper
 *
 * This class provides a simple interface to save supporter signup information
 * to your Salsa Labs account.
 *
 * @author      New Media Campaigns <http://www.newmediacampaigns.com>
 * @copyright   2011 New Media Campaigns
 * @license     MIT Public License
 *
 * USAGE
 *
 * $salsa = Salsa::getInstance();
 * $salsa->save(array(...));
 * $responseText = $salsa->getLastResponse();
 * $responseHeaders = $salsa->getLastResponseHeaders();
 *
 * REQUIREMENTS
 *
 * Salsa Labs account node URL
 * Salsa Labs account organization key
 * PHP >= 5.0.0
 * libcurl
 *
 * MIT PUBLIC LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
class Salsa {

    /*******************************************************
     * CHANGE THESE SETTINGS!
     ******************************************************/

    /**
     * @var string Salsa API node URL
     */
    private $node = 'http://your.node.url.tld'; //<-- Enter your node URL here

    /**
     * @var string Salsa API organization key
     */
    private $orgKey = 'ABC123'; // <-- Enter your organization key here

    /**
     * @var array Map your database columns (array keys) to Salsa API database columns (array values)
     */
    private $keyMap = array(
        'created_at' => 'Date_Created',
        'name_first' => 'First_Name',
        'name_last' => 'Last_Name',
        'email' => 'Email',
        'phone' => 'Cell_Phone',
        'phone_home' => 'Phone',
        'city' => 'City',
        'state' => 'State',
        'zip' => 'Zip',
        'unit_company' => 'Organization',
        'id' => 'uid'
    );

    /*******************************************************
     * DO NOT EDIT BELOW THIS LINE!
     ******************************************************/

    /**
     * @var Salsa Singleton instance
     */
    private static $instance;

    /**
     * @var string Last API response
     */
    private $lastResponse = false;

    /**
     * @var array Last API response headers
     */
    private $lastResponseHeaders = false;

    /**
     * Constructor (Disabled)
     */
    private function __construct() {}

    /**
     * Get singleton instance
     * @return Salsa
     */
    public static function getInstance() {
        if ( is_null(self::$instance) ) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Save user object to Salsa API
     * @param array Key/value data for user object
     * @return bool
     */
    public function save($userData, $debug = false) {
        //Build POST url
        $url = $this->node . '/save?json';

        //Build POST body
        $body = '&object=supporter&key=0&organization_KEY=' . $this->orgKey;
        foreach ( $userData as $key => $value ) {
            if ( isset($this->keyMap[$key]) ) {
                $salsaKey = $this->keyMap[$key];
                $body .= sprintf('&%s=%s', $salsaKey, urlencode($value));
            }
        }

        //Send POST request
        if ( $debug ) {
            exit("Sending POST request to Salsa API at $url with body:\n$body");
        } else {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url . $body);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Length: 0'));
            curl_setopt($ch, CURLOPT_POSTFIELDS, '');
            $this->lastResponse = curl_exec($ch);
            $this->lastResponseHeaders = curl_getinfo($ch);
            curl_close($ch);
        }
    }

    /**
     * Get last API response
     * @return string|false
     */
    public function getLastResponse() {
        return $this->lastResponse;
    }

    /**
     * Get last API response headers
     * @return array|false
     */
    public function getLastResponseHeaders() {
        return $this->lastResponseHeaders;
    }
}
?>